package com.ruoyi.business.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.business.domain.ServiceGrant;
import com.ruoyi.common.auth.entity.OauthClientDetails;
import com.ruoyi.system.domain.vo.ServiceGrantVo;

import java.util.List;

/**
 * 对外服务授权Service接口
 * 
 * @author wzq
 * @date 2024-01-10
 */
public interface IServiceGrantService extends IService<ServiceGrant> {

    /**
     * 查询对外服务授权列表
     * 
     * @param outGrant 对外服务授权
     * @return 对外服务授权集合
     */
    List<ServiceGrant> selectServiceGrantList(ServiceGrant outGrant);

    /**
     * 查询对外服务授权详情
     *
     * @param id 对外服务授权主键
     * @return 对外服务授权
     */
     ServiceGrant selectServiceGrantById(Long id);

    /**
     * 新增对外服务授权
     * 
     * @param outGrant 对外服务授权
     * @return 结果
     */
    int insertServiceGrant(ServiceGrant outGrant);

    /**
     * 修改对外服务授权
     * 
     * @param outGrant 对外服务授权
     * @return 结果
     */
    Boolean updateServiceGrant(ServiceGrant outGrant);

    /**
     * 批量删除对外服务授权
     * 
     * @param ids 需要删除的对外服务授权主键集合
     * @return 结果
     */
    int deleteServiceGrantByIds(Long[] ids);

    /**
     * 删除对外服务授权信息
     * 
     * @param id 对外服务授权主键
     * @return 结果
     */
    int deleteServiceGrantById(Long id);

    /**
     * 对外服务授权
     *
     * @param outGrant 对外服务授权
     * @return 结果
     */
    int updateGrant(ServiceGrantVo outGrant);

    /**
     * 根据客户端id查询菜单树ids信息
     *
     * @param clientId 客户端id
     * @return 选中菜单列表
     */
    public List<String> selectServiceBusinessIdByClientId(String clientId, String belongSystem);

    String generateDocument(OauthClientDetails oAuthClientDetails);
}
